#!/usr/bin/env python

from libvtkCommonPython import *
from libvtkGraphicsPython import *
from libvtkRenderingPython import *
from vtk.tk.vtkTkRenderWidget import *
from Tkinter import *
from string import *
import sys
import os
import tkMessageBox
import time


## ----------------- Only for make a comboBox button  ----------------

class DropDown(Frame):
    "Generic drop-down menu using radiobuttons"

    def __init__ ( self, parent, var, choiceList ):
        "Constructor for a DropDown widget"
        Frame.__init__ ( self, parent, relief=RAISED, borderwidth=2 )
        self.var         =  var
        self.choiceList  =  choiceList
        self.__index     =  0
        self.__mb  =  Menubutton ( self, text=choiceList[0] + "  V", font="-*-Courier-bold-r-*-*-12")
        self.__mb.grid ( row=0, column=0 )
        self.__menu  =  Menu ( self.__mb )
        self.__rbList  =  []
        for  i in range(len(choiceList)):
            choice  =  choiceList[i]
            def handler(s=self, i=i):
                self.__index  =  i
                self.__mb["text"]  =  self.var.get() + "  V"
            rb  =  self.__menu.add_radiobutton ( label=choice,
                value=choice, variable=self.var, command=handler )
            self.__rbList.append ( rb )
        self.__mb["menu"]  =  self.__menu

    def set ( self, i ):
        "Set self to the (i)th choice."
        value  =  self.choiceList[i]
        self.var.set ( self.choiceList[i] )
        self.__mb["text"]  =  self.choiceList[i]
        self.__index       =  i

    def get ( self ):
        "Return the index of the currently selected entry"
        return self.__index

## ----------------- The application class ----------------

class Application:
    "show VTK actors in a TK window"

    def __init__(self,master):	
	self.lastsearch = []
	self.actors = []
	self.sequences = []
	self.ren = vtkRenderer()
	self.ren.SetBackground(0.1,0.2,0.4)
	self.renderWidget = vtkTkRenderWidget(master,width=550,height=550)
	self.renderWidget.pack(expand='true',fill='both')
	self.renWin = self.renderWidget.GetRenderWindow()
	self.renWin.AddRenderer(self.ren)
	self.showAllMessage = 1 
	self.firstShow=1
	
	# main button menu 
	frame = Frame(master,bd=2,relief=RAISED)
	frame.pack()

	bt1 = Button(frame, text="Actor", command=self.mnActor)
	bt2 = Button(frame, text="Color", command=self.mnColor)
	bt3 = Button(frame, text="Photo", command=self.mnPhoto)
	bt4 = Button(frame, text="Video", command=self.mnVideo)
	bt5 = Button(frame, text="Size", command=self.mnSize)
	bt6 = Button(frame, text="Exec", command=self.mnExec)

	bt7 = Button(frame, text="Axis", command=self.showAxis)
	bt8 = Button(frame, text="Reset", command=self.setCamera)
	bt9 = Button(frame, text="Quit", command=frame.quit)

	bt1.grid(row=0,column=0,sticky=W+E)
	bt2.grid(row=0,column=1,sticky=W+E)
	bt3.grid(row=0,column=2,sticky=W+E)
	bt4.grid(row=0,column=3,sticky=W+E)
	bt5.grid(row=0,column=4,sticky=W+E)
	bt6.grid(row=0,column=5,sticky=W+E)
	bt7.grid(row=0,column=6,sticky=W+E)
	bt8.grid(row=0,column=7,sticky=W+E)
	bt9.grid(row=0,column=8,sticky=W+E)
	
	self.btActor = 0
	self.btColor = 0
	self.btPhoto = 0
	self.btVideo = 0
	self.btSize = 0
	self.btAxis = 0
	self.btExec = 0

	# Make the actor frame
	self.frActor = Frame(frame,bd=2,relief='ridge')
	Label(self.frActor, text=" --- Actor Menu --- \n").grid(row=0,columnspan=6)
	Label(self.frActor, text="Criteria: ").grid(row=1,sticky=E)
	self.txObjID1 = Entry(self.frActor, width=24)
	self.txObjID1.grid(row=1,column=1,sticky=W)
	self.cbFrAc1 = IntVar()
	Label(self.frActor, text=" ").grid(row=2,column=2)
	cbFrAc = Checkbutton(self.frActor,text="by step ",variable=self.cbFrAc1)
	cbFrAc.grid(row=1,column=3)
	self.txStep1 = Entry(self.frActor, width=4)
	self.txStep1.grid(row=1,column=4)
	Label(self.frActor, text=" seg").grid(row=1,column=5,sticky=W)
	Button(self.frActor, text=" Show ", command=self.bSAc).grid(row=3,column=2,columnspan=2)
	Button(self.frActor, text="Hidden", command=self.bHAc).grid(row=3,column=4,columnspan=2)
	Label(self.frActor, text='(examples of criteria: " all ", " 1==7 ", " 1<=9; 1!=5 ")      ').grid(row=3,columnspan=2,sticky=W)
	

	# Make the color frame
	self.frColor = Frame(frame,bd=2,relief='ridge')
	Label(self.frColor, text=" --- Color Menu --- \n").grid(row=0,columnspan=7)
	Label(self.frColor, text="Criteria: ").grid(row=1,sticky=E)
	self.txObjID2 = Entry(self.frColor, width=14)
	self.txObjID2.grid(row=1,column=1,sticky=W)
	Label(self.frColor, text="  Color: ").grid(row=1,column=2,sticky=E)
	self.txColRGB = Entry(self.frColor, width=10)
	self.txColRGB.grid(row=1,column=3,sticky=W)
	Label(self.frColor, text=" ").grid(row=2,column=4)
	self.cb1FrCol = IntVar()
	cb1FrCo = Checkbutton(self.frColor,text="Trans",variable=self.cb1FrCol)
	cb1FrCo.grid(row=1,column=5, sticky=E)
	self.tx1frCol = Entry(self.frColor, width=4)
	self.tx1frCol.grid(row=1,column=6, sticky=W)
	Button(self.frColor,text=" Ok ",command=self.bCCo).grid(row=3,column=6)
	Label(self.frColor, text='(examples of color: " 0.5,0.1,0.1 ", trans: " 0.5 ")          ').grid(row=3,columnspan=4,sticky=W)


	# Make the photo frame
	self.frPhoto = Frame(frame,bd=2,relief='ridge')
	Label(self.frPhoto, text=" --- Photo Menu --- \n").grid(row=0,columnspan=6)
	Label(self.frPhoto, text="filename: ").grid(row=1,column=0,sticky=E)
	self.txPhIfn = Entry(self.frPhoto, width=12)
	self.txPhIfn.grid(row=1,column=1,sticky=W)
	Label(self.frPhoto, text="       format: ").grid(row=1,column=2,sticky=E)
	formats = [".bmp ", ".jpg ", ".png ", ".tif ", ".pnm ", ".ps  "]
	self.comboPhFr = StringVar()
        comboPh = DropDown(self.frPhoto, self.comboPhFr, formats)
	comboPh.grid(row=1,column=3)
	Label(self.frPhoto, text="     ").grid(row=1,column=4)
	Button(self.frPhoto, text=" OK ", command=self.bTP).grid(row=1,column=5)


	# Make the video frame
	self.frVideo = Frame(frame,bd=2,relief='ridge')
	Label(self.frVideo, text=" --- Video Menu --- \n").grid(row=0,columnspan=7)
	Label(self.frVideo, text="Sequence: ").grid(row=1,column=0,sticky=E+N)
	scrollbar = Scrollbar(self.frVideo,orient=VERTICAL)
	self.listVid = Listbox(self.frVideo,height=3,selectmode=BROWSE,yscrollcommand=scrollbar.set)
	scrollbar.config(command=self.listVid.yview)
	scrollbar.grid(row=1,column=2,rowspan=3,sticky=W+N+S)
	self.listVid.grid(row=1,column=1,rowspan=3,sticky=W+N+S)
	self.radioVid = StringVar()
	r = Radiobutton(self.frVideo, text=" gif", variable=self.radioVid, value=".gif")
	r.grid(row=1,column=3,sticky=W)
	r.select()
	Radiobutton(self.frVideo, text=" mpg", variable=self.radioVid, value=".mpg").grid(row=2,column=3,sticky=W)
	Label(self.frVideo, text="       Delay: ").grid(row=1,column=4,sticky=W)
	self.txVidDel = Entry(self.frVideo, width=5)
	self.txVidDel.grid(row=1,column=5,sticky=W)
	Label(self.frVideo, text="       Factor: ").grid(row=2,column=4,sticky=W)
	self.txVidFac = Entry(self.frVideo, width=4)
	self.txVidFac.grid(row=2,column=5,sticky=W)
	Button(self.frVideo, text=" OK ", command=self.bVideo).grid(row=4,column=5,sticky=E)


	# Make the size frame
	self.frSize = Frame(frame,bd=2,relief='ridge')
	Label(self.frSize, text=" --- Size Menu --- \n").grid(row=0,columnspan=7)
	Label(self.frSize, text="Image size: ").grid(row=1,columnspan=2,sticky=W)
	self.txSize = Entry(self.frSize, width=12)
	self.txSize.grid(row=1,column=2,columnspan=2,sticky=W)
	Label(self.frSize, text=" ex: 640x480  ").grid(row=1,column=4,sticky=W)
	Button(self.frSize, text=" Ok ", command=self.changeSize).grid(row=1,column=6)


	# Make the exec frame
	self.frExec = Frame(frame,bd=2,relief='ridge')
	Label(self.frExec, text=" --- Exec Menu --- ").grid(row=0,columnspan=5)
	msg="Some Objects:\n\n"
	msg=msg+"self.ren      (renderer)\nself.renWin   (ren. Window)\nself.actors[] (Boxs Actors)"
	text1 = Text(self.frExec,bd=0,width=28,height=5,relief='flat')
	text1.insert(END, msg)
	text1.config(state='disabled')
	text1.grid(row=1,rowspan=3,sticky=W+N)
	Label(self.frExec, text="   Command: ").grid(row=1,column=1,sticky=W)
	self.txComText = Entry(self.frExec, width=22)
	self.txComText.grid(row=1,column=2,sticky=W)
	Button(self.frExec, text=" OK ", command=self.execCommand).grid(row=1,column=4,columnspan=2)
	Label(self.frExec, text="   Execfile: ").grid(row=2,column=1,sticky=W)
	self.txComFile = Entry(self.frExec, width=20)
	self.txComFile.grid(row=2,column=2,sticky=W)
	Button(self.frExec, text="Load", command=self.execFile).grid(row=2,column=4,columnspan=2)
	Label(self.frExec, text="   results").grid(row=3,column=1,columnspan=5,sticky=W)
	scrollbar2 = Scrollbar(self.frExec,orient=VERTICAL)
	self.txRes = Text(self.frExec, width=70, height=4,yscrollcommand=scrollbar2.set,state='disable')
	scrollbar2.config(command=self.txRes.yview)
	self.txRes.grid(row=4, column=0,columnspan=5,sticky=E)
	scrollbar2.grid(row=4,column=5,sticky=W+N+S)
	Label(self.frExec, text="  ").grid(row=5,column=3)

    # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Show/Hidden menu frames in the application
    # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

    def hiddenAll(self):
	"hidden all frames from application"
	self.frActor.grid_forget()
	self.btActor = 0
	self.frColor.grid_forget()
	self.btColor = 0
	self.frPhoto.grid_forget()
	self.btPhoto = 0
	self.frVideo.grid_forget()
	self.btVideo = 0
	self.frSize.grid_forget()
	self.btSize = 0
	self.frExec.grid_forget()
	self.btExec = 0

    def mnActor(self):
	"Show/Hidden the actor menu"
	if self.btActor == 0:
	    self.hiddenAll()
	    self.frActor.grid(row=2,columnspan=9,sticky=W+E)
	else:
	    self.frActor.grid_forget()
	self.btActor = 1 - self.btActor	

    def mnColor(self):
	"Show/Hidden the color menu"
	if self.btColor == 0:
	    self.hiddenAll()
	    self.frColor.grid(row=2,columnspan=9,sticky=W+E)
	else:
	    self.frColor.grid_forget()
	self.btColor = 1 - self.btColor	    

    def mnPhoto(self):
	"Show/Hidden the photo menu"
	if self.btPhoto == 0:
	    self.hiddenAll()
	    self.frPhoto.grid(row=2,columnspan=9,sticky=W+E)
	else:
	    self.frPhoto.grid_forget()
	self.btPhoto = 1 - self.btPhoto	

    def mnVideo(self):
	"Show/Hidden the video menu"
	if self.btVideo == 0:
	    if self.sequences != []:
		self.listVid.delete(0,END)
		for item in self.sequences:
		    self.listVid.insert(END, item[0])
		self.listVid.select_set(0)
	    self.hiddenAll()		
	    self.frVideo.grid(row=2,columnspan=9,sticky=W+E)
	else:
	    self.frVideo.grid_forget()
	self.btVideo = 1 - self.btVideo	

    def mnSize(self):
	"Show/Hidden the size menu"
	if self.btSize == 0:
	    self.hiddenAll()
	    self.frSize.grid(row=2,columnspan=9,sticky=W+E)
	else:
	    self.frSize.grid_forget()
	self.btSize = 1 - self.btSize	

    def mnExec(self):
	"Show/Hidden the exec menu"
	if self.btExec == 0:
	    self.hiddenAll()
	    self.frExec.grid(row=2,columnspan=9,sticky=W+E)
	else:
	    self.frExec.grid_forget()
	self.btExec = 1 - self.btExec	

    # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Command buttons (for make specifics task)
    # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

    def setCamera(self):
	"Reset the camera from initial values"
	if self.firstShow < 1:
	    self.ren.GetActiveCamera().SetClippingRange(self.MyCam[0])
	    self.ren.GetActiveCamera().SetFocalPoint(self.MyCam[1])
	    self.ren.GetActiveCamera().SetPosition(self.MyCam[2])
	    self.ren.GetActiveCamera().SetViewUp(self.MyCam[3])
	    if self.btAxis == 1 and self.firstShow == 0:
		self.ren.ResetCamera()
	    self.renWin.Render()

    def makeActors(self, b):
	"Make the actors for scene"
	self.b = b
	for i in b.boxes:
	    cube = vtkCubeSource()
	    if b.dim==2:
		cube.SetBounds(i[0],i[1],i[2],i[3],0,0.0001)
	    else:
		cube.SetBounds(i[0],i[1],i[2],i[3],i[4],i[5])
	    cubeMapper = vtkPolyDataMapper()
	    cubeMapper.SetInput(cube.GetOutput())
	    cubeActor = vtkActor()
	    cubeActor.SetMapper(cubeMapper)
	    cubeActor.GetProperty().SetColor(0.5,0.2,0.2)
	    self.actors.append(cubeActor)
	self.maxAxs = b.max * 1.1
	self.initAxis()

    def showAxis(self):
	"Show/Hidden axis in scena"
	if self.firstShow==1:
	    self.firstShow=-1
	    self.ren.AddActor(self.axisActors[0])
	    self.ren.ResetCamera()
	    cam = self.ren.GetActiveCamera()
	    self.MyCam = [cam.GetClippingRange(), cam.GetFocalPoint(), cam.GetPosition(), cam.GetViewUp()]
	    for i in self.axisActors[1:]:
		self.ren.AddActor(i)
		i.SetCamera(cam)
	    self.renWin.Render()
	    self.btAxis=1
	else:
	    if self.btAxis == 0:
		self.ren.AddActor(self.axisActors[0])
		camera = self.ren.GetActiveCamera()
		for i in self.axisActors[1:]:
		    self.ren.AddActor(i)
		    i.SetCamera(camera)
		self.renWin.Render()
	    else:
		for i in self.axisActors:
		    self.ren.RemoveActor(i)
		self.renWin.Render()
	    self.btAxis = 1 - self.btAxis


    def initAxis(self):
	"Make the axis"
	self.axisInfo = [0,self.maxAxs,1,0,self.maxAxs,1,0,self.maxAxs,1]

	axes = vtkAxes()
	axes.SetOrigin(0, 0, 0)
	axes.SetScaleFactor(self.maxAxs)
	
	axesTubes = vtkTubeFilter()
	axesTubes.SetInput(axes.GetOutput())
	axesTubes.SetRadius(axes.GetScaleFactor()/100.0)
	axesTubes.SetNumberOfSides(6)
    
	axesMapper = vtkPolyDataMapper()
	axesMapper.SetInput(axesTubes.GetOutput())

	axesActor = vtkActor()
	axesActor.SetMapper(axesMapper)

	# Label the axes.
	XText = vtkVectorText()
	XText.SetText("X")

	XTextMapper = vtkPolyDataMapper()
	XTextMapper.SetInput(XText.GetOutput())

	XActor = vtkFollower()
	XActor.SetMapper(XTextMapper)
	XActor.SetScale(0.25, 0.25, 0.25)
	XActor.SetPosition(axes.GetScaleFactor()*1.1, 0, 0)
	XActor.GetProperty().SetColor(1, 0, 0)
    
	YText = vtkVectorText()
	YText.SetText("Y")
    
	YTextMapper = vtkPolyDataMapper()
	YTextMapper.SetInput(YText.GetOutput())
	
	YActor = vtkFollower()
	YActor.SetMapper(YTextMapper)
	YActor.SetScale(0.25, 0.25, 0.25)
	YActor.SetPosition(0,axes.GetScaleFactor()*1.1,0)
	YActor.GetProperty().SetColor(1, 1, 0)
	
	ZText = vtkVectorText()
	ZText.SetText("Z")
	
	ZTextMapper = vtkPolyDataMapper()
	ZTextMapper.SetInput(ZText.GetOutput())
	
	ZActor = vtkFollower()
	ZActor.SetMapper(ZTextMapper)
	ZActor.SetScale(0.25, 0.25, 0.25)
	ZActor.SetPosition(0, 0, axes.GetScaleFactor()*1.1)
	ZActor.GetProperty().SetColor(0, 1, 0)
	
	self.axisActors = [axesActor, XActor, YActor, ZActor]

    def makeCriterion(self, text):
	"Make a list of criteria for selecting objects"
	self.list = []
	findbox=find(text,'box')
	err = "Illegal objects criteria"
	if findbox!=-1:
	    findend=find(text,')')
	    i=text[findbox:findend+1]
	    text=text[:findbox]+text[findend+1:]
	    boxcrit = ['box']
	    for j in range(self.b.dim):
		i=strip(i[find(i,"[")+1:])
		boxcrit.append(float((i[0:find(i,",")])))
		boxcrit.append(float((i[find(i,",")+1: find(i,"]")])))
	    self.list.append(boxcrit)
	elif find(lower(text),'all')!=-1:
	    self.list.append(['all'])
	    return 1

	all = split(strip(text), ';')
	for i in all:
	    if strip(i) != '':
		pos=find(i,'<')
		if pos==-1:
		    pos=find(i,'>')
		    if pos==-1:
			pos=find(i,'!')
			if pos==-1:
			    pos=find(i,'=')
			    if pos==-1:
				if self.showAllMessage == 1:
				    tkMessageBox.showwarning("Bad input", err)
				return 0
		criter = []
		criter.append(int(strip(i[:pos])))
		if i[pos+1]=='=':
		    criter.append(strip(i[pos:pos+2]))
		    pos=pos+1
		else:
		    criter.append(i[pos])
		criter.append(int(i[pos+1:]))
		self.list.append(criter)
	if self.list == []:
	    if self.showAllMessage == 1:
		tkMessageBox.showwarning("Bad input", err)
	    return 0
	return 1

    def selectObjects(self):
	"Select objects from criteria"
	boxcrit = []
	if self.list == self.lastsearch:
	    return 1
	if self.list[0][0] == 'all':
	    self.searchResult = range(len(self.b.boxes))
	    return 1
	elif self.list[0][0] == 'box':
	    boxcrit = self.list[0][1:]
	    self.list = self.list[1:]
	self.lastsearch = self.list
	self.searchResult = []
	for i in range(len(self.b.boxes)):
	    allOK = 1
	    if boxcrit != []:
		for k in range(self.b.dim):
		    if self.b.boxes[i][2*k]<boxcrit[2*k] or  self.b.boxes[i][2*k+1]>boxcrit[2*k+1]:
			allOK = 0
	    for crit in self.list:
		if allOK == 1:
		    pos = self.b.dim * 2 + crit[0] - 1
		    if crit[1] == ">":
			if not(self.b.boxes[i][pos] > crit[2]):
			    allOK = 0
		    elif crit[1] == ">=":
			if not(self.b.boxes[i][pos] >= crit[2]):
			    allOK = 0
		    elif crit[1] == "<":
			if not(self.b.boxes[i][pos] < crit[2]):
			    allOK = 0
		    elif crit[1] == "<=":
			if not(self.b.boxes[i][pos] <= crit[2]):
			    allOK = 0
		    elif crit[1] == "!=":
			if not(self.b.boxes[i][pos] != crit[2]):
			    allOK = 0
		    elif crit[1] == "==" or crit[1] == "=":
			if not(self.b.boxes[i][pos] == crit[2]):
			    allOK = 0		    
	    if allOK == 1:
		self.searchResult.append(i)
	if self.searchResult == []:
	    if self.showAllMessage == 1:
		tkMessageBox.showwarning("Not Found", "There are not objects with criteria")
	    return 0
	return 1

    def writephoto(self,image,filename):
	"Write the image parameter into disk with the filename"
	if filename[0]!=".":
	    renSRC=vtkRendererSource()
	    renSRC.SetInput(self.ren)
	    image.SetInput(renSRC.GetOutput())
	    image.SetFileName(filename)
	    image.Write()
	    if self.showAllMessage == 1:
		tkMessageBox.showinfo("Task completed", filename+" wrote!")
	    return 1
	else:
	    tkMessageBox.showwarning("Bad input", "No filename specified")
	    return 0

    def changeSize(self):
	"Change the image window size"
	size = split(strip(self.txSize.get()),'x')
	if len(size)==2:
	    if int(size[0])>=550:
		self.renderWidget.configure(width=int(size[0]),height=int(size[1]))
		self.renWin.Render()
	    else:
		tkMessageBox.showwarning("Bad input", "The minimun value for X is 550")
	else:
	    tkMessageBox.showwarning("Bad input", "The format is 'value' x 'value'")

    # -=-=-=-=-=-=-=-=-=-=-=-
    # Operations from frames
    # -=-=-=-=-=-=-=-=-=-=-=-

    def bSAc(self):
	"Show actors on window"
	if self.makeCriterion(self.txObjID1.get()) and self.selectObjects():
	    for actor in self.searchResult:
		self.ren.AddActor(self.actors[actor])
		if self.cbFrAc1.get() == 1 and strip(self.txStep1.get())!="":
		    self.renWin.Render()
		    time.sleep(float(self.txStep1.get()))
	    if self.firstShow==1:
		self.firstShow=0
		self.ren.ResetCamera()
		cam = self.ren.GetActiveCamera()
		self.MyCam = [cam.GetClippingRange(), cam.GetFocalPoint(), cam.GetPosition(), cam.GetViewUp()]
	    self.renWin.Render()

    def bHAc(self):
	"Hidden actors from window"
	if self.makeCriterion(self.txObjID1.get()) and self.selectObjects():
	    for actor in self.searchResult:
		self.ren.RemoveActor(self.actors[actor])
		if self.cbFrAc1.get() == 1 and strip(self.txStep1.get())!="":
		    self.renWin.Render()
		    time.sleep( float(self.txStep1.get()))		    
	    self.renWin.Render()
		    
    def bCCo(self):
	"Change actors color"
	cRGB = split(strip(self.txColRGB.get()), ",")
	if strip(self.txObjID2.get()) == 'bg' or strip(self.txObjID2.get())== 'background':
	    if cRGB != ['']:
		self.ren.SetBackground(float(cRGB[0]),float(cRGB[1]),float(cRGB[2]))
		self.renWin.Render()
	elif self.makeCriterion(self.txObjID2.get()) and self.selectObjects():
	    for actor in self.searchResult:
		if cRGB != ['']:
		    self.actors[actor].GetProperty().SetColor(float(cRGB[0]),float(cRGB[1]),float(cRGB[2]))
		if self.cb1FrCol.get() == 1:
		    self.actors[actor].GetProperty().SetOpacity(float(self.tx1frCol.get()))
	    self.renWin.Render()

    def bTP(self):
	"Take a photo and write it on disk"
	misel = strip(self.comboPhFr.get())
	if misel =="":
	    misel = ".bmp"
	image = vtkImageWriter()
	if misel == ".bmp":
	    image=vtkBMPWriter()
	elif misel == ".jpg":
	    image=vtkJPEGWriter()
	elif misel == ".png":
	    image=vtkPNGWriter()
	elif misel == ".tif":
	    image=vtkTIFFWriter()
	elif misel == ".pnm":
	    image=vtkPNMWriter()
	elif misel == ".ps":
	    image=vtkPostScriptWriter()
	filename = strip(self.txPhIfn.get())
	for i in range(len(self.sequences)):
	    if filename == self.sequences[i][0]:
		filename = '_'+filename+"_/"+filename + zfill(str(self.sequences[i][1]),3)
		self.sequences[i][1] = self.sequences[i][1] + 1
	try:
	    f = open(filename+misel, 'r')
	    msg = "The filename already exists!\nDo you want to make a sequence?"
	    if tkMessageBox.askyesno("Error", msg):
		self.renWin.Render()
		try:
		    os.mkdir('_'+filename+'_')
		except OSError:
		    pass
		os.rename(filename+misel,  '_'+filename+'_/'+filename+'000'+misel)
		self.sequences.append([filename, 2])
		self.writephoto(image,'_'+filename+'_/'+filename+'001' + misel)
		del image
		return
	    else:
		tkMessageBox.showwarning("Error", "The image file not wrote.")
		return		
	except IOError:
	    pass
	self.writephoto(image,filename + misel)
	del image


    def bVideo(self):
	"Make a video file from images"
	items = self.listVid.curselection()
	if items == ():
	    tkMessageBox.showwarning("Error", "No sequence selected!")
	else:
	    filename = self.listVid.get(items[0])
	    run = "convert"
	    if strip(self.txVidDel.get()) != "":
		run = run + "  -delay " + str(float(self.txVidDel.get())*100)
	    if strip(self.txVidFac.get()) != "":
		run = run + "  -resize " + str(float(self.txVidFac.get())*100) + "%"
	    run = run + " _" + filename + "_/" + filename  + "???.* " + filename + self.radioVid.get()
	    # os.system(run)
	    if self.radioVid.get() == ".gif":
		view = "xanim " + filename + self.radioVid.get()
	    else:
		view = "mplayer " + filename + self.radioVid.get()
	    tkMessageBox.showinfo("Task in background", "The task will run in background mode!")
	    os.system("(" + run + "; " + view + ")&")


    def execCommand(self):
	"Exec a command"
	try:
	    exec(self.txComText.get())
	    self.txRes.config(state='normal')
	    self.txRes.insert(END, self.txComText.get()+" --> OK!\n")
	    self.txRes.see(END)
	    self.txRes.config(state='disabled')
	    self.renWin.Render()
	except:
	    self.txRes.config(state='normal')
	    self.txRes.insert(END, self.txComText.get() + " -> Error ("+ str(sys.exc_info()[1]) +")\n")
	    self.txRes.see(END)
	    self.txRes.config(state='disabled')
	    

    def execFile(self):
	"Exec a File"
	if strip(self.txComFile.get()) != "":
	    self.showAllMessage = 0
	    try:
		execfile(strip(self.txComFile.get()))
		self.txRes.config(state='normal')
		self.txRes.insert(END, "Execfile(" + self.txComFile.get()+") --> OK!\n")
		self.txRes.see(END)
		self.txRes.config(state='disabled')
		self.renWin.Render()
	    except:
		self.txRes.config(state='normal')
		self.txRes.insert(END, "Execfile(" + self.txComFile.get() + ") -> Error ("+ str(sys.exc_info()[1]) +")\n")
		self.txRes.see(END)
		self.txRes.config(state='disabled')
	    self.showAllMessage = 1
	else:
	    tkMessageBox.showwarning("Error", "filename field empty") 

class Boxes:
    "storage all boxes for painting"

    def __init__(self):
	self.boxes = []
	self.dim = 0
	self.max = 1.0

    def readFile(self, name):
	"read boxes information from file"
	f=open(name)
	lines=f.readlines()
	f.close()
	self.dim = count(lines[0],';') + 1
	for i in lines:
	    if strip(i)!="":
		box = []
		for j in range(self.dim):
		    i=strip(i[find(i,"[")+1:])
		    box.append(float((i[0:find(i,",")])))
		    max = float((i[find(i,",")+1: find(i,"]")]))
		    box.append(max)
		    if max > self.max:
			self.max = max
		pps = split(strip(split(i, ')')[1]), ' ')
		for j in pps:
		    if strip(i)!='':
			box.append(int(j))
		self.boxes.append(box)
		del box
	del lines


if __name__ == "__main__":
    if len(sys.argv) <> 2:
        print "Use: <program_name> <file_to_read>\n"
        sys.exit()
    else:
	b = Boxes()
	b.readFile(sys.argv[1])
	root = Tk()
	root.title("MAPA v0.1 by cgrandon")
	root.resizable(0,0)
	a = Application(root)
	a.makeActors(b)
	root.mainloop()














